'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var loaderUtils = require('loader-utils');
var path = require('path');
var path__default = _interopDefault(path);
var lodash = require('lodash');
var ora = _interopDefault(require('ora'));
var prettyMs = _interopDefault(require('pretty-ms'));
var fs = require('fs');
var deepMerge = _interopDefault(require('deepmerge'));
var ExtractTextPlugin = _interopDefault(require('extract-text-webpack-plugin'));
var webpack = _interopDefault(require('webpack'));
var StatsPlugin = _interopDefault(require('stats-webpack-plugin'));
var AssetsPlugin = _interopDefault(require('assets-webpack-plugin'));
var nodeExternals = _interopDefault(require('webpack-node-externals'));
var DuplicateCheckerPlugin = _interopDefault(require('duplicate-package-checker-webpack-plugin'));
var HardSourceWebpackPlugin = _interopDefault(require('hard-source-webpack-plugin'));
var webpackBundleAnalyzer = require('webpack-bundle-analyzer');
var CaseSensitivePathsPlugin = _interopDefault(require('case-sensitive-paths-webpack-plugin'));
var ChunkManifestPlugin = _interopDefault(require('chunk-manifest-webpack-plugin'));

class WatchMissingNodeModulesPlugin {
  constructor(nodeModulesPath) {
    this.nodeModulesPath = nodeModulesPath;
  }

  apply(compiler) {
    compiler.plugin('emit', (compilation, callback) => {
      var missingDeps = compilation.missingDependencies;
      var nodeModulesPath = this.nodeModulesPath;

      if (missingDeps.some(file => file.indexOf(nodeModulesPath) !== -1)) {
        compilation.contextDependencies.push(nodeModulesPath);
      }

      callback();
    });
  }
}

var WatchMissingNodeModulesPlugin_1 = WatchMissingNodeModulesPlugin;

var matchOperatorsRe = /[|\\{}()[\]^$+*?.]/g;

var escapeStringRegexp = function escapeStringRegexp(str) {
	if (typeof str !== 'string') {
		throw new TypeError('Expected a string');
	}

	return str.replace(matchOperatorsRe, '\\$&');
};

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var ansiStyles = createCommonjsModule(function (module) {

	function assembleStyles() {
		var styles = {
			modifiers: {
				reset: [0, 0],
				bold: [1, 22],
				dim: [2, 22],
				italic: [3, 23],
				underline: [4, 24],
				inverse: [7, 27],
				hidden: [8, 28],
				strikethrough: [9, 29]
			},
			colors: {
				black: [30, 39],
				red: [31, 39],
				green: [32, 39],
				yellow: [33, 39],
				blue: [34, 39],
				magenta: [35, 39],
				cyan: [36, 39],
				white: [37, 39],
				gray: [90, 39]
			},
			bgColors: {
				bgBlack: [40, 49],
				bgRed: [41, 49],
				bgGreen: [42, 49],
				bgYellow: [43, 49],
				bgBlue: [44, 49],
				bgMagenta: [45, 49],
				bgCyan: [46, 49],
				bgWhite: [47, 49]
			}
		};

		styles.colors.grey = styles.colors.gray;

		Object.keys(styles).forEach(function (groupName) {
			var group = styles[groupName];

			Object.keys(group).forEach(function (styleName) {
				var style = group[styleName];

				styles[styleName] = group[styleName] = {
					open: '\u001b[' + style[0] + 'm',
					close: '\u001b[' + style[1] + 'm'
				};
			});

			Object.defineProperty(styles, groupName, {
				value: group,
				enumerable: false
			});
		});

		return styles;
	}

	Object.defineProperty(module, 'exports', {
		enumerable: true,
		get: assembleStyles
	});
});

var ansiRegex = function ansiRegex() {
	return (/[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-PRZcf-nqry=><]/g
	);
};

var ansiRegex$1 = ansiRegex();

var stripAnsi = function stripAnsi(str) {
	return typeof str === 'string' ? str.replace(ansiRegex$1, '') : str;
};

var re = new RegExp(ansiRegex().source);
var hasAnsi = re.test.bind(re);

var argv = process.argv;

var terminator = argv.indexOf('--');
var hasFlag = function hasFlag(flag) {
	flag = '--' + flag;
	var pos = argv.indexOf(flag);
	return pos !== -1 && (terminator !== -1 ? pos < terminator : true);
};

var supportsColor = function () {
	if ('FORCE_COLOR' in process.env) {
		return true;
	}

	if (hasFlag('no-color') || hasFlag('no-colors') || hasFlag('color=false')) {
		return false;
	}

	if (hasFlag('color') || hasFlag('colors') || hasFlag('color=true') || hasFlag('color=always')) {
		return true;
	}

	if (process.stdout && !process.stdout.isTTY) {
		return false;
	}

	if (process.platform === 'win32') {
		return true;
	}

	if ('COLORTERM' in process.env) {
		return true;
	}

	if (process.env.TERM === 'dumb') {
		return false;
	}

	if (/^screen|^xterm|^vt100|color|ansi|cygwin|linux/i.test(process.env.TERM)) {
		return true;
	}

	return false;
}();

var defineProps = Object.defineProperties;
var isSimpleWindowsTerm = process.platform === 'win32' && !/^xterm/i.test(process.env.TERM);

function Chalk(options) {
	this.enabled = !options || options.enabled === undefined ? supportsColor : options.enabled;
}

if (isSimpleWindowsTerm) {
	ansiStyles.blue.open = '\u001b[94m';
}

var styles = function () {
	var ret = {};

	Object.keys(ansiStyles).forEach(function (key) {
		ansiStyles[key].closeRe = new RegExp(escapeStringRegexp(ansiStyles[key].close), 'g');

		ret[key] = {
			get: function get() {
				return build.call(this, this._styles.concat(key));
			}
		};
	});

	return ret;
}();

var proto = defineProps(function chalk() {}, styles);

function build(_styles) {
	var builder = function builder() {
		return applyStyle.apply(builder, arguments);
	};

	builder._styles = _styles;
	builder.enabled = this.enabled;

	builder.__proto__ = proto;

	return builder;
}

function applyStyle() {
	var args = arguments;
	var argsLen = args.length;
	var str = argsLen !== 0 && String(arguments[0]);

	if (argsLen > 1) {
		for (var a = 1; a < argsLen; a++) {
			str += ' ' + args[a];
		}
	}

	if (!this.enabled || !str) {
		return str;
	}

	var nestedStyles = this._styles;
	var i = nestedStyles.length;

	var originalDim = ansiStyles.dim.open;
	if (isSimpleWindowsTerm && (nestedStyles.indexOf('gray') !== -1 || nestedStyles.indexOf('grey') !== -1)) {
		ansiStyles.dim.open = '';
	}

	while (i--) {
		var code = ansiStyles[nestedStyles[i]];

		str = code.open + str.replace(code.closeRe, code.open) + code.close;
	}

	ansiStyles.dim.open = originalDim;

	return str;
}

function init() {
	var ret = {};

	Object.keys(styles).forEach(function (name) {
		ret[name] = {
			get: function get() {
				return build.call(this, [name]);
			}
		};
	});

	return ret;
}

defineProps(Chalk.prototype, init());

var chalk = new Chalk();
var styles_1 = ansiStyles;
var hasColor = hasAnsi;
var stripColor = stripAnsi;
var supportsColor_1 = supportsColor;
chalk.styles = styles_1;
chalk.hasColor = hasColor;
chalk.stripColor = stripColor;
chalk.supportsColor = supportsColor_1;

var name = "frack-build-config-factory";
var version = "0.8.1-beta.14";
var description = "Utility scripts to create opinionated webpack and rollup configuration.";
var license = "MIT";
var repository = "https://git.siteone.cz/frack/frack-build-config-factory.git";
var main = "lib/node.classic.commonjs.js";
var module$1 = "lib/node.classic.esmodule.js";
var es2015 = "lib/node.es2015.esmodule.js";
var browser = "lib/browser.classic.esmodule.js";
var author = {
	name: "Viktor Bezdek",
	email: "viktor.bezdek@siteone.cz",
	url: "siteone.cz"
};
var engines = {
	node: ">=6.0.0",
	yarn: ">=0.21.3",
	npm: ">=5.0.0"
};
var files = ["lib", "src"];
var scripts = {
	build: "rollup -c",
	dev: "rollup -c -w",
	prepublish: "npm run build",
	coverage: "npm test -- --coverage",
	postcoverage: "opn coverage/lcov-report/index.html",
	lint: "eslint ./src/** ./test --fix",
	flow: "flow check",
	docs: "documentation readme src --section=API",
	clean: "rimraf lib",
	sprebuild: "npm run docs && npm run clean",
	watch: "npm-watch",
	beta: "npm version prerelease && npm publish",
	patch: "npm version patch && npm publish",
	minor: "npm version minor && npm publish",
	major: "npm version major && npm publish",
	test: "npm run lint && npm test && npm run build",
	postpublish: "git push origin master --follow-tags"
};
var watch = {
	test: "{src,test}/*.js",
	lint: "{src,test}/*.js",
	build: "src"
};
var jest = {
	testRegex: "/test/.*",
	testEnvironment: "node"
};
var keywords = ["generator-nod"];
var dependencies = {
	"assets-webpack-plugin": "^3.9.6",
	"babel-core": "^6.26.3",
	"babel-eslint": "^10.0.0",
	"babel-jest": "^23.6.0",
	"babel-loader": "^7.1.5",
	"babel-plugin-dynamic-import-node": "^2.1.0",
	"babel-plugin-external-helpers": "^6.22.0",
	"babel-plugin-flow-react-proptypes": "^24.1.2",
	"babel-plugin-import-inspector": "^2.0.0",
	"babel-plugin-jsx-control-statements": "^3.2.8",
	"babel-plugin-log-deprecated": "^1.1.0",
	"babel-plugin-macros": "^2.4.2",
	"babel-plugin-module-resolver": "^3.1.1",
	"babel-plugin-syntax-dynamic-import": "^6.18.0",
	"babel-plugin-syntax-flow": "^6.18.0",
	"babel-plugin-transform-class-properties": "^6.24.1",
	"babel-plugin-transform-flow-strip-types": "^6.22.0",
	"babel-plugin-transform-object-assign": "^6.22.0",
	"babel-plugin-transform-object-rest-spread": "^6.26.0",
	"babel-plugin-transform-react-constant-elements": "^6.23.0",
	"babel-plugin-transform-react-inline-elements": "^6.22.0",
	"babel-plugin-transform-react-jsx-self": "^6.22.0",
	"babel-plugin-transform-react-jsx-source": "^6.22.0",
	"babel-plugin-transform-react-remove-prop-types": "^0.4.18",
	"babel-plugin-transform-runtime": "^6.23.0",
	"babel-polyfill": "^6.26.0",
	"babel-preset-env": "^1.7.0",
	"babel-preset-flow": "^6.23.0",
	"babel-preset-react": "^6.24.1",
	"babel-preset-react-app": "^3.1.2",
	"babel-preset-stage-0": "^6.24.1",
	bluebird: "^3.5.2",
	"case-sensitive-paths-webpack-plugin": "^2.1.2",
	"chunk-manifest-webpack-plugin": "^1.1.2",
	"css-loader": "^1.0.0",
	deepmerge: "^2.1.1",
	"duplicate-package-checker-webpack-plugin": "^2.1.0",
	"eslint-loader": "^2.1.1",
	"extract-text-webpack-plugin": "^3.0.2",
	"fast-sass-loader": "^1.4.6",
	"git-rev-sync": "^1.12.0",
	"hard-source-webpack-plugin": "^0.12.0",
	"import-inspector": "^2.0.0",
	"jsx-control-statements": "^3.2.8",
	"loader-utils": "^1.1.0",
	lodash: "^4.17.11",
	"lodash-webpack-plugin": "^0.11.5",
	"node-sass": "^4.9.3",
	ora: "^3.0.0",
	"postcss-loader": "^3.0.0",
	"pretty-ms": "^4.0.0",
	"react-dev-utils": "^5.0.2",
	"stats-webpack-plugin": "^0.7.0",
	"style-loader": "^0.23.0",
	"svg-react-loader": "^0.4.5",
	"webpack-bundle-analyzer": "^3.0.2",
	"webpack-chunk-hash": "^0.5.0",
	"webpack-node-externals": "^1.7.2"
};
var devDependencies = {
	documentation: "8.1.2",
	"flow-bin": "^0.81.0",
	"frack-settings": "^0.1.10",
	"jest-cli": "^23.6.0",
	moment: "^2.22.2",
	"npm-watch": "^0.4.0",
	"opn-cli": "^3.1.0",
	react: "^16.5.2",
	rimraf: "^2.6.2",
	rollup: "^0.66.2",
	"rollup-plugin-babel": "^3.0.7",
	"rollup-plugin-commonjs": "^9.1.8",
	"rollup-plugin-flow": "^1.1.1",
	"rollup-plugin-json": "^3.1.0",
	"rollup-plugin-node-builtins": "^2.1.2",
	"rollup-plugin-node-resolve": "^3.4.0",
	"rollup-watch": "^4.3.1",
	shelljs: "^0.8.2",
	webpack: "^3.12.0"
};
var configPkg = {
	name: name,
	version: version,
	description: description,
	license: license,
	repository: repository,
	main: main,
	module: module$1,
	es2015: es2015,
	"main:modern": "lib/node.modern.commonjs.js",
	"module:modern": "lib/node.modern.esmodule.js",
	browser: browser,
	"browser:modern": "lib/browser.modern.esmodule.js",
	author: author,
	engines: engines,
	files: files,
	scripts: scripts,
	watch: watch,
	jest: jest,
	keywords: keywords,
	dependencies: dependencies,
	devDependencies: devDependencies
};

function compareModules(left, right) {
  if (left.resource < right.resource) {
    return -1;
  }
  if (left.resource > right.resource) {
    return 1;
  }
  return 0;
}

function getModuleSource(module) {
  const source = module._source || {};
  return source._value || '';
}

function concatenateSource(result, moduleSource) {
  return result + moduleSource;
}

function ChunkHash() {}

const hashType = 'sha256';
const digestType = 'base62';
const digestLength = 8;

ChunkHash.prototype.apply = function (compiler) {
  compiler.plugin('compilation', compilation => {
    compilation.plugin('chunk-hash', (chunk, chunkHash) => {
      const source = chunk.modules.sort(compareModules).map(getModuleSource).reduce(concatenateSource, '');

      const generatedHash = loaderUtils.getHashDigest(source, hashType, digestType, digestLength);
      chunkHash.digest = function () {
        return generatedHash;
      };
    });
  });
};

const WORKING_DIR = process.cwd();
const HASH_TYPE = 'sha256';
const DIGEST_TYPE = 'base62';
const DIGEST_LENGTH = 4;
const SCRIPT_EXTENSIONS = new Set(['.mjs', '.js', '.jsx', '.ts', '.tsx']);
const SKIP_FOLDERS = ['lib', 'dist', 'build'];

function generateChunkName(request, rawRequest) {
  const splittedRequest = request.split('!');
  const cleanRequest = splittedRequest[splittedRequest.length - 1];

  let relative = path__default.relative(WORKING_DIR, cleanRequest);

  const isExternal = relative.startsWith(`node_modules${path__default.sep}`);
  if (isExternal) {
    relative = rawRequest || request;
  }

  SKIP_FOLDERS.forEach(filter => {
    relative = relative.replace(new RegExp(`(^|/|\\\\)${filter}($|/|\\\\)`), (match, group1) => group1);
  });

  const fileExt = path__default.parse(relative).ext;
  if (SCRIPT_EXTENSIONS.has(fileExt)) {
    relative = relative.replace(new RegExp(`${fileExt}$`), '');
  }

  const hash = loaderUtils.getHashDigest(cleanRequest, HASH_TYPE, DIGEST_TYPE, DIGEST_LENGTH);
  const base = path__default.basename(relative);

  const result = `${base}-${hash}`;

  return result;
}

class ChunkNames {
  apply(compiler) {
    compiler.plugin('compilation', compilation => {
      compilation.plugin('optimize', () => {
        compilation.chunks.forEach(chunk => {
          const firstModule = chunk.mapModules(c => c)[0];
          if (firstModule) {
            const userRequest = firstModule.userRequest;
            const rawRequest = firstModule.rawRequest;
            const oldName = chunk.name;
            if (userRequest && oldName == null) {
              chunk.name = generateChunkName(userRequest, rawRequest);
            }
          }
        });
      });
    });
  }
}

class VerboseProgress {
  constructor(options) {
    this.options = null;

    this.options = options;
  }

  apply(compiler) {
    let modules;
    let seal;
    let optimize;
    let optimizeAssets;
    let emit;

    compiler.plugin('compilation', compilation => {
      if (compilation.compiler.isChild()) {
        return;
      }

      let moduleCounter = 0;

      compilation.plugin('build-module', module => {
        const ident = module.identifier();
        if (ident) {
          if (lodash.startsWith(ident, 'ignored') || lodash.startsWith(ident, 'external')) {
            return;
          }
          if (moduleCounter === 0) {
            modules = ora({ text: 'Building module', spinner: 'clock' }).start();
          }
          moduleCounter += 1;
          if (moduleCounter % 10 === 0) {
            modules.text = `Bundling module #${moduleCounter}`;
          }
        }
      });

      compilation.plugin('seal', () => {
        modules && modules.succeed();
        seal = ora({ text: `Sealing ${moduleCounter} modules`, spinner: 'clock' });
      });

      compilation.plugin('optimize', () => {
        seal && seal.succeed();
        optimize = ora({ text: 'Optimizing modules', spinner: 'clock' }).start();
      });

      compilation.plugin('optimize-chunk-assets', (chunks, callback) => {
        optimize && optimize.succeed();
        optimizeAssets = ora({ text: 'Optimizing assets', spinner: 'clock' }).start();
        callback();
      });

      compilation.plugin('after-optimize-chunk-assets', (chunks, callback) => {
        optimizeAssets && optimizeAssets.succeed();
      });
    });

    compiler.plugin('emit', (compilation, callback) => {
      optimizeAssets.succeed();
      emit = ora({ text: 'Emitting output files', spinner: 'clock' }).start();
      callback();
    });

    compiler.plugin('after-emit', (compilation, callback) => {
      emit && emit.succeed();
      callback();
    });

    compiler.plugin('done', stats => {
      ora({
        text: `Compilation took ${prettyMs(stats.endTime - stats.startTime)}\n`
      }).stopAndPersist('🤘');
    });
  }
}

function removeEmpty(array) {
  return array.filter(entry => Boolean(entry));
}

function removeEmptyKeys(obj) {
  const copy = {};
  for (const key in obj) {
    if (!(obj[key] == null || obj[key].length === 0)) {
      copy[key] = obj[key];
    }
  }

  return copy;
}

function ifElse(condition) {
  return (then, otherwise) => condition ? then : otherwise;
}

const root = path.resolve(__dirname, '..', 'node_modules');
const Modules = new Set();

const nodePackages = fs.readdirSync(root).filter(dirname => dirname.charAt(0) !== '.');

nodePackages.forEach(packageName => {
  try {
    var json = JSON.parse(fs.readFileSync(path.resolve(root, packageName, 'package.json')));
  } catch (error) {
    return;
  }

  if (json.module || json.style || json['jsnext:main']) {
    Modules.add(packageName);
  }
});

const BABEL_COMMON = {
  babelrc: false,

  compact: true,

  sourceMap: true,

  comments: false,

  cacheDirectory: true,

  presets: [require.resolve('babel-preset-stage-0'), require.resolve('babel-preset-flow'), require.resolve('babel-preset-react')],

  plugins: [[require.resolve('babel-plugin-macros'), require.resolve('babel-plugin-transform-runtime'), {
    regenerator: true,
    polyfill: true,
    useBuiltIns: true,
    useESModules: false
  }], require.resolve('babel-plugin-jsx-control-statements'), [require.resolve('babel-plugin-import-inspector'), {
    serverSideRequirePath: true,
    webpackRequireWeakId: true
  }], [require.resolve('babel-plugin-transform-regenerator'), {
    async: false
  }], require.resolve('babel-plugin-syntax-dynamic-import'), require.resolve('babel-plugin-transform-class-properties'), [require.resolve('babel-plugin-transform-object-rest-spread'), { useBuiltIns: true }]],

  env: {
    production: {
      comments: false,
      plugins: [require.resolve('babel-plugin-jsx-control-statements'), [require.resolve('babel-plugin-transform-react-remove-prop-types'), { removeImport: true }]]
    },
    development: {
      plugins: [require.resolve('babel-plugin-jsx-control-statements'), require.resolve('babel-plugin-log-deprecated')]
    }
  }
};

const BABEL_NODE = {
  presets: [[require('babel-preset-env').default, {
    targets: {
      node: 8.0,
      uglify: true
    },

    modules: false
  }]],
  plugins: [require.resolve('babel-plugin-dynamic-import-node')]
};

const BABEL_WEB = {
  presets: [[require.resolve('babel-preset-env'), {
    targets: {
      safari: '9.1.3',

      ie: 11,

      uglify: true
    },

    useBuiltIns: false,

    modules: false
  }]]
};

function getJsLoader({
  isNode, isWeb, babelConfig = {}, root
}) {
  const resolverPlugin = {
    plugins: [['module-resolver', {
      root: [path.resolve(root, 'src')]
    }]]
  };
  const commonBabel = deepMerge(resolverPlugin, BABEL_COMMON);
  const nodeBabel = isNode ? BABEL_NODE : {};
  const webBabel = isWeb ? BABEL_WEB : {};
  const config = [{
    loader: 'babel-loader',
    options: deepMerge(deepMerge(nodeBabel, webBabel), deepMerge(babelConfig, commonBabel))
  }];

  return config;
}

const modulesPlugin = new ExtractTextPlugin({
  filename: 'modules-[contenthash:base62:8].css',
  allChunks: true
});
const globalsPlugin = new ExtractTextPlugin({
  filename: 'global-[contenthash:base62:8].css',
  allChunks: true
});

function getCssLoaders(options) {
  const isNode = options.isNode,
        isWeb = options.isWeb,
        isProd = options.isProd;
  var _options$enableModule = options.enableModules;
  const enableModules = _options$enableModule === undefined ? true : _options$enableModule;

  const moduleIdentName = enableModules ? '[name]__[local]' : '[local]';

  if (isNode) {
    return [{
      loader: 'css-loader/locals',
      query: {
        sourceMap: false,
        modules: enableModules,
        localIdentName: moduleIdentName,
        importLoaders: 2,
        minimize: false
      }
    }, {
      loader: 'postcss-loader',
      query: {
        sourceMap: false
      }
    }, {
      loader: 'fast-sass-loader',
      query: {
        sourceMap: false
      }
    }];
  }
  if (isWeb) {
    if (isProd) {
      return (enableModules ? modulesPlugin : globalsPlugin).extract({
        allChunks: true,
        fallback: 'style-loader',
        filename: 'css/[name]-[contenthash].css',
        use: [{
          loader: 'css-loader',
          query: {
            modules: enableModules,
            localIdentName: moduleIdentName,
            minimize: false,
            importLoaders: 2,
            sourceMap: false
          }
        }, {
          loader: 'postcss-loader',
          query: {
            sourceMap: false
          }
        }, {
          loader: 'fast-sass-loader',
          query: {
            sourceMap: false
          }
        }]
      });
    }

    return [{
      loader: 'style-loader'
    }, {
      loader: 'css-loader',
      query: {
        sourceMap: false,
        modules: enableModules,
        localIdentName: moduleIdentName,
        minimize: false,
        importLoaders: 2
      }
    }, {
      loader: 'postcss-loader',
      query: {
        sourceMap: false
      }
    }, {
      loader: 'fast-sass-loader',
      query: {
        sourceMap: false
      }
    }];
  }
  return [];
}

const cache = {
  'web-production': {},
  'web-development': {},
  'node-production': {},
  'node-development': {}
};

function configFactory(options) {
  const target = options.target,
        mode = options.mode,
        output = options.output;
  var _options$root = options.root;
  const root = _options$root === undefined ? process.cwd() : _options$root;
  var _options$isDebug = options.isDebug;
  const isDebug = _options$isDebug === undefined ? false : _options$isDebug;
  var _options$isVerbose = options.isVerbose;
  const isVerbose = _options$isVerbose === undefined ? false : _options$isVerbose;
  var _options$entry = options.entry;
  const entry = _options$entry === undefined ? {} : _options$entry,
        settings = options.settings;

  console.info('\n\n\n', `Build config version ${configPkg.version} targetting ${target}-${mode}`, '\n\n\n');

  const enableHardSource = !settings.disableHardsource;

  if (!target || !['web', 'node'].includes(target)) {
    throw new Error('You must provide a "target" (web|node) to the configFactory.');
  }
  if (!mode || !['development', 'production'].includes(mode)) {
    throw new Error('You must provide a "mode" (development|production) to the configFactory.');
  }
  if (settings.clientBundleOutputPath === undefined) {
    throw new Error("No .frackrc.js file found. Please provide one to your project's root. ");
  }

  ora(`Creating bundle for ${target} in ${mode} mode.`).stopAndPersist('🤘');
  settings.nodeEnv = isDebug ? 'development' : mode;
  settings.babelEnv = mode;
  const version$$1 = JSON.stringify(require(path__default.join(root, 'package.json')).version);
  const isDev = mode === 'development';
  const isProd = mode === 'production';
  const isWeb = target === 'web';
  const isNode = target === 'node';
  const ifDev = ifElse(isDev);
  const ifProd = ifElse(isProd);
  const ifWeb = ifElse(isWeb);
  const ifNode = ifElse(isNode);
  const ifDevWeb = ifElse(isDev && isWeb);
  const ifProdWeb = ifElse(isProd && isWeb);
  const folder = isWeb ? 'client' : 'server';
  const CONFIG = {
    isProd,
    isDev,
    isWeb,
    isNode,
    root
  };
  const cssLoadersWithoutModules = getCssLoaders(Object.assign({}, CONFIG, { enableModules: false }));
  const cssLoadersWithModules = getCssLoaders(Object.assign({}, CONFIG, { enableModules: true }));
  const jsLoaders = getJsLoader(CONFIG);
  const excludeFromTranspilation = [/node_modules/, path__default.resolve(root, settings.clientBundleOutputPath), path__default.resolve(root, settings.serverBundleOutputPath)];
  return {
    target: ifNode('node', 'web'),

    node: {
      __dirname: true,
      __filename: true,
      process: true
    },

    stats: isDebug || isVerbose ? {
      colors: true,
      reasons: isDebug,
      hash: isVerbose,
      version: isVerbose,
      timings: true,
      chunks: isVerbose,
      chunkModules: isVerbose,
      cached: isVerbose,
      cachedAssets: isVerbose
    } : {
      reasons: true,
      colors: true
    },

    cache: cache[`${target}-${mode}`],

    profile: isProd,

    bail: isProd,
    externals: nodeExternals(),

    devtool: 'source-map',

    performance: isProd && isWeb ? { hints: 'warning' } : false,

    entry: Object.keys(entry).length > 0 ? entry : removeEmptyKeys({
      main: removeEmpty(['babel-polyfill', ifDevWeb(`webpack-hot-middleware/client?overlay=true&path=http://localhost:${settings.clientDevserverPort}/__webpack_hmr`), `./src/${folder}/index.js`]),
      config: ifProdWeb('./src/application/config.js')
    }),
    output: Object.assign({
      path: path__default.resolve(root, isWeb ? settings.clientBundleOutputPath : settings.serverBundleOutputPath),

      filename: ifProdWeb(`[name]-[chunkhash]-${version$$1.replace(/[^0-9]/g, '')}.js`, '[name].js'),
      chunkFilename: ifProdWeb('[name]-[chunkhash].js', '[name].js'),

      publicPath: ifDev(`http://localhost:${settings.clientDevserverPort}${settings.clientBundleHttpPath}`, settings.clientBundleHttpPath),

      libraryTarget: ifNode('commonjs2', 'var')
    }, output),
    recordsOutputPath: path__default.resolve(settings.clientBundleOutputPath, 'records.json'),
    resolve: {
      mainFields: ifNode(enableHardSource ? ['main'] : ['module', 'jsnext:main', 'main'], enableHardSource ? ['web', 'browser', 'style', 'main'] : ['web', 'browser', 'style', 'module', 'jsnext:main', 'main']),
      modules: [path__default.resolve(root, 'node_modules'), path__default.resolve(root, 'src/application'), path__default.resolve(root, 'public')],
      alias: {
        'global-styles': path__default.resolve(root, 'src/application/global-styles'),
        application: path__default.resolve(root, 'src/application'),
        utils: path__default.resolve(root, 'src/application/utils'),
        store: path__default.resolve(root, 'src/application/store'),
        client: path__default.resolve(root, 'src/client'),
        server: path__default.resolve(root, 'src/server'),
        components: path__default.resolve(root, 'src/application/components'),
        modules: path__default.resolve(root, 'src/application/modules'),
        routes: path__default.resolve(root, 'src/application/routes'),
        services: path__default.resolve(root, 'src/application/services'),
        queries: path__default.resolve(root, 'src/application/queries'),
        public: path__default.resolve(root, 'public')
      },

      extensions: ['.js', '.jsx', '.json']
    },
    plugins: removeEmpty([new CaseSensitivePathsPlugin(), ifDev(new HardSourceWebpackPlugin({})), ifDev(new webpack.LoaderOptionsPlugin({
      minimize: false,
      debug: isDebug
    })), ifProd(new webpack.LoaderOptionsPlugin({
      minimize: true,
      debug: isDebug
    })), new VerboseProgress(), ifNode(new webpack.BannerPlugin({
      banner: 'require("source-map-support").install();',
      raw: true,
      entryOnly: false
    })), ifProdWeb(new webpack.optimize.CommonsChunkPlugin({
      name: 'vendor',
      minChunks(module) {
        return module.context && module.context.indexOf('node_modules') !== -1;
      }
    })), ifProdWeb(new webpack.optimize.AggressiveMergingPlugin()), ifProdWeb(new ChunkHash()), ifProdWeb(new ChunkManifestPlugin({
      filename: 'manifest.json',
      manifestVariable: 'CHUNK_MANIFEST'
    })), ifProdWeb(new ChunkNames()), new webpack.ContextReplacementPlugin(/moment[/\\]locale$/, /cs/), new webpack.DefinePlugin({
      'process.env.APP_ENV': JSON.stringify(process.env.APP_ENV),
      'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV),

      'process.env.API_ENDPOINT_REST_BASE': JSON.stringify(settings.apiEndpointRestBase),
      'process.env.BASE_URL': JSON.stringify(settings.baseUrl),
      'process.env.USE_REST_PROXY': JSON.stringify(settings.useRestProxy),
      'process.env.IS_BROWSER': isWeb,
      'process.env.VERSION': version$$1,
      'process.env.TARGET': JSON.stringify(target),
      'process.env.MODE': JSON.stringify(mode),
      'process.env.SERVER_PORT': settings.serverPort,
      'process.env.CLIENT_DEVSERVER_PORT': settings.clientDevserverPort,
      'process.env.DISABLE_SSR': settings.disableSsr,
      'process.env.SERVER_BUNDLE_OUTPUT_PATH': JSON.stringify(settings.serverBundleOutputPath),
      'process.env.CLIENT_BUNDLE_OUTPUT_PATH': JSON.stringify(settings.clientBundleOutputPath),
      'process.env.CLIENT_PUBLIC_PATH': JSON.stringify(settings.clientPublicPath),
      'process.env.CLIENT_BUNDLE_ASSETS_FILENAME': JSON.stringify(settings.clientBundleAssetsFilename),
      'process.env.CLIENT_BUNDLE_CHUNK_MANIFEST_FILENAME': JSON.stringify(settings.clientBundleChunkManifestFilename),
      'process.env.CLIENT_BUNDLE_HTTP_PATH': JSON.stringify(settings.clientBundleHttpPath),
      'process.env.CLIENT_BUNDLE_CACHE_MAXAGE': JSON.stringify(settings.clientBundleCacheMaxage)
    }), ifWeb(new AssetsPlugin({
      filename: 'assets.json',
      path: path__default.join(root, settings.clientBundleOutputPath),
      prettyPrint: true,
      update: false
    })), ifProdWeb(new StatsPlugin('stats.json')), ifDev(new webpack.NoEmitOnErrorsPlugin()), ifDevWeb(new webpack.HotModuleReplacementPlugin()), ifProdWeb(modulesPlugin), ifProdWeb(globalsPlugin), ifProdWeb(new DuplicateCheckerPlugin()), ifProdWeb(new webpackBundleAnalyzer.BundleAnalyzerPlugin({
      openAnalyzer: false,
      analyzerMode: 'static',
      logLevel: 'error'
    })), ifProdWeb(new webpack.optimize.UglifyJsPlugin({
      compress: { drop_console: true },
      sourceMap: true,
      mangle: true,
      warnings: false,
      comments: false,
      parallel: { cache: true, workers: 4 }
    })), new webpack.ProvidePlugin({
      Promise: 'bluebird'
    }), new WatchMissingNodeModulesPlugin_1(path__default.resolve(process.cwd(), 'node_modules'))]),
    module: {
      rules: removeEmpty([{
        test: /\.(svg)$/,
        include: [/src/, /assets/],
        loader: 'babel-loader?cacheDirectory=true,presets[]=es2015,presets[]=react!svg-react-loader'
      }, {
        test: /\.(mjs|js|jsx)$/,
        loaders: jsLoaders,
        exclude: excludeFromTranspilation
      }, {
        test: /\.mdx?$/,
        use: jsLoaders.concat(['mdx-loader'])
      }, {
        test: /[A-Z][A-z0-9]+\.(scss|css|sss|pcss)$/,
        loader: cssLoadersWithModules,
        include: [/src/],
        exclude: [/node_modules/, /global-styles/]
      }, {
        test: /[a-z][A-z0-9]+\.(scss|css|sss|pcss)$/,
        loader: cssLoadersWithoutModules,
        include: [/node_modules/, /src/, /global-styles/],
        exclude: [/components/]
      }, {
        test: /\.json$/,
        loader: 'json-loader',
        exclude: []
      }, {
        test: /\.(yml|yaml)$/,
        loaders: ['json-loader', 'yaml-loader']
      }, {
        test: /\.(eot|woff|woff2|ttf|otf|png|jpg|jpeg|jp2|jpx|jxr|gif|webp|mp4|mp3|ogg|pdf|html)$/,
        loader: 'file-loader',
        options: {
          name: ifProdWeb('file-[hash:base62:8].[ext]', '[name].[ext]'),
          emitFile: isWeb
        }
      }, {
        test: /\.(graphql|gql)$/,
        loader: 'graphql-tag/loader',
        exclude: excludeFromTranspilation
      }])
    }
  };
}

class Status {
  constructor(opts) {
    this.checkDone = () => {
      this.running = false;
      this.done();
    };

    const name = opts.name,
          start = opts.start,
          done = opts.done;

    this.name = name;

    this.start = start;
    this.done = done;

    this.watcher = null;
    this.running = false;
  }

  apply(compiler) {
    compiler.plugin('done', stats => {
      if (!stats.hasErrors()) {
        this.watcher = setTimeout(this.checkDone, 100);
      }
    });

    compiler.plugin('compile', stats => {
      if (this.watcher) {
        clearTimeout(this.watcher);
      }

      if (!this.running) {
        this.running = true;
        this.start();
      }
    });
  }
}

exports.configFactory = configFactory;
exports.ChunkHashPlugin = ChunkHash;
exports.ChunkNamesPlugin = ChunkNames;
exports.StatusPlugin = Status;
exports.VerboseProgressPlugin = VerboseProgress;
